@@ -254,13 +254,6 @@ type IssuedCertificate struct {
 
 type Definition map[string]interface{}
 
-type MachineInit struct {
-	Exec       []string `json:"exec"`
-	Entrypoint []string `json:"entrypoint"`
-	Cmd        []string `json:"cmd"`
-	Tty        bool     `json:"tty"`
-}
-
 func DefinitionPtr(in map[string]interface{}) *Definition {
 	x := Definition(in)
 	return &x
