@@ -1,6 +1,7 @@
 package app
 
 import (
+	"context"
 	"testing"
 
 	"github.com/BurntSushi/toml"
@@ -10,7 +11,7 @@ import (
 func TestLoadTOMLAppConfigWithAppName(t *testing.T) {
 	const path = "./testdata/app-name.toml"
 
-	p, err := LoadConfig(path)
+	p, err := LoadConfig(context.Background(), path, NomadPlatform)
 	assert.NoError(t, err)
 	assert.Equal(t, p.AppName, "test-app")
 }
@@ -18,7 +19,7 @@ func TestLoadTOMLAppConfigWithAppName(t *testing.T) {
 func TestLoadTOMLAppConfigWithBuilderName(t *testing.T) {
 	const path = "./testdata/build.toml"
 
-	p, err := LoadConfig(path)
+	p, err := LoadConfig(context.Background(), path, NomadPlatform)
 	assert.NoError(t, err)
 	assert.Equal(t, p.Build.Builder, "builder/name")
 }
@@ -26,7 +27,7 @@ func TestLoadTOMLAppConfigWithBuilderName(t *testing.T) {
 func TestLoadTOMLAppConfigWithImage(t *testing.T) {
 	const path = "./testdata/image.toml"
 
-	p, err := LoadConfig(path)
+	p, err := LoadConfig(context.Background(), path, NomadPlatform)
 	assert.NoError(t, err)
 	assert.Equal(t, p.Build.Image, "image/name")
 }
@@ -34,7 +35,7 @@ func TestLoadTOMLAppConfigWithImage(t *testing.T) {
 func TestLoadTOMLAppConfigWithDockerfile(t *testing.T) {
 	const path = "./testdata/docker.toml"
 
-	p, err := LoadConfig(path)
+	p, err := LoadConfig(context.Background(), path, NomadPlatform)
 	assert.NoError(t, err)
 	assert.Equal(t, p.Build.Dockerfile, "./Dockerfile")
 }
@@ -42,14 +43,14 @@ func TestLoadTOMLAppConfigWithDockerfile(t *testing.T) {
 func TestLoadTOMLAppConfigWithBuilderNameAndArgs(t *testing.T) {
 	const path = "./testdata/build-with-args.toml"
 
-	p, err := LoadConfig(path)
+	p, err := LoadConfig(context.Background(), path, NomadPlatform)
 	assert.NoError(t, err)
 	assert.Equal(t, p.Build.Args, map[string]string{"A": "B", "C": "D"})
 }
 
 func TestLoadTOMLAppConfigWithServices(t *testing.T) {
 	const path = "./testdata/services.toml"
-	p, err := LoadConfig(path)
+	p, err := LoadConfig(context.Background(), path, NomadPlatform)
 
 	rawData := map[string]interface{}{}
 	toml.DecodeFile("./testdata/services.toml", &rawData)
