@@ -14,12 +14,14 @@ import (
 
 	"github.com/AlecAivazis/survey/v2"
 	"github.com/logrusorgru/aurora"
+	"github.com/samber/lo"
 	"github.com/spf13/cobra"
 	"github.com/superfly/flyctl/api"
 	"github.com/superfly/flyctl/client"
 	"github.com/superfly/flyctl/flyctl"
 	"github.com/superfly/flyctl/helpers"
 	"github.com/superfly/flyctl/internal/app"
+	"github.com/superfly/flyctl/internal/appv2"
 	"github.com/superfly/flyctl/internal/build/imgsrc"
 	"github.com/superfly/flyctl/internal/command"
 	"github.com/superfly/flyctl/internal/command/deploy"
@@ -41,7 +43,6 @@ func New() (cmd *cobra.Command) {
 	)
 
 	cmd = command.New("launch", short, long, run, command.RequireSession, command.LoadAppConfigIfPresent)
-
 	cmd.Args = cobra.NoArgs
 
 	flag.Add(cmd,
@@ -76,6 +77,11 @@ func New() (cmd *cobra.Command) {
 			Description: "If a .dockerignore does not exist, create one from .gitignore files",
 			Default:     false,
 		},
+		flag.Int{
+			Name:        "internal-port",
+			Description: "Set internal_port for all services in the generated fly.toml",
+			Default:     -1,
+		},
 	)
 
 	return
@@ -91,36 +97,35 @@ func run(ctx context.Context) (err error) {
 		workingDir = absDir
 	}
 
+	var importedConfig bool
 	appConfig := app.NewConfig()
 
-	var importedConfig bool
 	configFilePath := filepath.Join(workingDir, "fly.toml")
-
 	if exists, _ := flyctl.ConfigFileExistsAtPath(configFilePath); exists {
 		cfg, err := app.LoadConfig(ctx, configFilePath, "nomad")
 		if err != nil {
 			return err
 		}
 
-		var deployExisting bool
-
 		if cfg.AppName != "" {
 			fmt.Fprintln(io.Out, "An existing fly.toml file was found for app", cfg.AppName)
-			deployExisting, err = shouldDeployExistingApp(ctx, cfg.AppName)
-			if err != nil {
+			if deployExisting, err := shouldDeployExistingApp(ctx, cfg.AppName); err != nil {
 				return err
+			} else if deployExisting {
+				fmt.Fprintln(io.Out, "App is not running, deploy...")
+				ctx = app.WithName(ctx, cfg.AppName)
+				ctx = appv2.WithName(ctx, cfg.AppName)
+				return deploy.DeployWithConfig(ctx, cfg, deploy.DeployWithConfigArgs{
+					ForceNomad:    flag.GetBool(ctx, "force-nomad"),
+					ForceMachines: flag.GetBool(ctx, "force-machines"),
+					ForceYes:      flag.GetBool(ctx, "now"),
+				})
 			}
 		} else {
 			fmt.Fprintln(io.Out, "An existing fly.toml file was found")
 		}
 
-		if deployExisting {
-			fmt.Fprintln(io.Out, "App is not running, deploy...")
-			return deploy.DeployWithConfig(ctx, cfg)
-		}
-
 		copyConfig := false
-
 		if flag.GetBool(ctx, "copy-config") {
 			copyConfig = true
 		} else {
@@ -131,7 +136,7 @@ func run(ctx context.Context) (err error) {
 		}
 
 		if copyConfig {
-			appConfig.Definition = cfg.Definition
+			appConfig = cfg
 			importedConfig = true
 		}
 	}
@@ -162,7 +167,6 @@ func run(ctx context.Context) (err error) {
 		}
 	} else {
 		fmt.Fprintln(io.Out, "Scanning source code")
-
 		if si, err := scanner.Scan(workingDir, config); err != nil {
 			return err
 		} else {
@@ -172,16 +176,12 @@ func run(ctx context.Context) (err error) {
 		if srcInfo == nil {
 			fmt.Fprintln(io.Out, aurora.Green("Could not find a Dockerfile, nor detect a runtime or framework from source code. Continuing with a blank app."))
 		} else {
-
 			var article string = "a"
-			matched, _ := regexp.MatchString(`^[aeiou]`, strings.ToLower(srcInfo.Family))
-
-			if matched {
+			if matched, _ := regexp.MatchString(`^[aeiou]`, strings.ToLower(srcInfo.Family)); matched {
 				article += "n"
 			}
 
 			appType := srcInfo.Family
-
 			if srcInfo.Version != "" {
 				appType = appType + " " + srcInfo.Version
 			}
@@ -206,7 +206,6 @@ func run(ctx context.Context) (err error) {
 	if srcInfo != nil {
 		for _, f := range srcInfo.Files {
 			path := filepath.Join(workingDir, f.Path)
-
 			if helpers.FileExists(path) {
 				if flag.GetBool(ctx, "now") {
 					fmt.Fprintf(io.Out, "You specified --now, so not overwriting %s\n", path)
@@ -223,7 +222,6 @@ func run(ctx context.Context) (err error) {
 			}
 
 			perms := 0o600
-
 			if strings.Contains(string(f.Contents), "#!") {
 				perms = 0o700
 			}
@@ -234,22 +232,16 @@ func run(ctx context.Context) (err error) {
 		}
 	}
 
-	// Prompt for an app name or fetch from flags
-	appName := ""
-
 	if !flag.GetBool(ctx, "generate-name") {
-		appName = flag.GetString(ctx, "name")
-
-		if appName == "" {
+		if appName := flag.GetString(ctx, "name"); appName == "" {
 			// Prompt the user for the app name
-			inputName, err := prompt.SelectAppName(ctx)
-			if err != nil {
+			if inputName, err := prompt.SelectAppName(ctx); err != nil {
 				return err
+			} else {
+				appConfig.AppName = inputName
 			}
-
-			appName = inputName
 		} else {
-			fmt.Fprintf(io.Out, "Selected App Name: %s\n", appName)
+			appConfig.AppName = appName
 		}
 	}
 
@@ -259,7 +251,6 @@ func run(ctx context.Context) (err error) {
 	if err != nil {
 		return
 	}
-
 	// If we potentially are deploying, launch a remote builder to prepare for deployment.
 	if !flag.GetBool(ctx, "no-deploy") {
 		go imgsrc.EagerlyEnsureRemoteBuilder(ctx, client, org.Slug)
@@ -268,13 +259,12 @@ func run(ctx context.Context) (err error) {
 	region, err := prompt.Region(ctx, prompt.RegionParams{
 		Message: "Choose a region for deployment:",
 	})
-
 	if err != nil {
 		return err
 	}
 
 	input := api.CreateAppInput{
-		Name:            appName,
+		Name:            appConfig.AppName,
 		OrganizationID:  org.ID,
 		PreferredRegion: &region.Code,
 	}
@@ -283,6 +273,8 @@ func run(ctx context.Context) (err error) {
 	if err != nil {
 		return err
 	}
+	ctx = app.WithName(ctx, createdApp.Name)
+	ctx = appv2.WithName(ctx, createdApp.Name)
 	if !importedConfig {
 		appConfig.Definition = createdApp.Config.Definition
 	}
@@ -298,12 +290,8 @@ func run(ctx context.Context) (err error) {
 	// If secrets are requested by the launch scanner, ask the user to input them
 	if srcInfo != nil && len(srcInfo.Secrets) > 0 {
 		secrets := make(map[string]string)
-		keys := []string{}
-
 		for _, secret := range srcInfo.Secrets {
-
 			val := ""
-
 			// If a secret should be a random default, just generate it without displaying
 			// Otherwise, prompt to type it in
 			if secret.Generate != nil {
@@ -314,35 +302,28 @@ func run(ctx context.Context) (err error) {
 				val = secret.Value
 			} else {
 				prompt := fmt.Sprintf("Set secret %s:", secret.Key)
-
-				surveyInput := &survey.Input{
-					Message: prompt,
-					Help:    secret.Help,
-				}
-
+				surveyInput := &survey.Input{Message: prompt, Help: secret.Help}
 				survey.AskOne(surveyInput, &val)
 			}
 
 			if val != "" {
 				secrets[secret.Key] = val
-				keys = append(keys, secret.Key)
 			}
 		}
 
 		if len(secrets) > 0 {
-			_, err := client.SetSecrets(ctx, createdApp.Name, secrets)
+			_, err := client.SetSecrets(ctx, appConfig.AppName, secrets)
 			if err != nil {
 				return err
 			}
-			fmt.Fprintf(io.Out, "Set secrets on %s: %s\n", createdApp.Name, strings.Join(keys, ", "))
+			fmt.Fprintf(io.Out, "Set secrets on %s: %s\n", appConfig.AppName, strings.Join(lo.Keys(secrets), ", "))
 		}
 	}
 
 	// If volumes are requested by the launch scanner, create them
 	if srcInfo != nil && len(srcInfo.Volumes) > 0 {
 		for _, vol := range srcInfo.Volumes {
-
-			appID, err := client.GetAppID(ctx, createdApp.Name)
+			appID, err := client.GetAppID(ctx, appConfig.AppName)
 			if err != nil {
 				return err
 			}
@@ -365,20 +346,16 @@ func run(ctx context.Context) (err error) {
 	}
 
 	options := make(map[string]bool)
-
-	if srcInfo != nil && !flag.GetBool(ctx, "no-deploy") && !flag.GetBool(ctx, "now") && !srcInfo.SkipDatabase {
-
+	if !(srcInfo == nil || srcInfo.SkipDatabase || flag.GetBool(ctx, "no-deploy") || flag.GetBool(ctx, "now")) {
 		confirmPg, err := prompt.Confirm(ctx, "Would you like to set up a Postgresql database now?")
-
 		if confirmPg && err == nil {
-			LaunchPostgres(ctx, createdApp, org, region)
+			LaunchPostgres(ctx, appConfig.AppName, org, region)
 			options["postgresql"] = true
 		}
 
 		confirmRedis, err := prompt.Confirm(ctx, "Would you like to set up an Upstash Redis database now?")
-
 		if confirmRedis && err == nil {
-			LaunchRedis(ctx, createdApp, org, region)
+			LaunchRedis(ctx, appConfig.AppName, org, region)
 			options["redis"] = true
 		}
 
@@ -392,7 +369,6 @@ func run(ctx context.Context) (err error) {
 				}
 			}
 		}
-
 	}
 
 	// Invoke Callback, if any
@@ -401,7 +377,6 @@ func run(ctx context.Context) (err error) {
 			return err
 		}
 	}
-
 	// Run any initialization commands
 	if srcInfo != nil && len(srcInfo.InitCommands) > 0 {
 		for _, cmd := range srcInfo.InitCommands {
@@ -416,7 +391,6 @@ func run(ctx context.Context) (err error) {
 
 	// Complete the appConfig
 	if srcInfo != nil {
-
 		if srcInfo.Port > 0 {
 			appConfig.SetInternalPort(srcInfo.Port)
 		}
@@ -431,18 +405,32 @@ func run(ctx context.Context) (err error) {
 
 		for envName, envVal := range srcInfo.Env {
 			if envVal == "APP_FQDN" {
-				appConfig.SetEnvVariable(envName, createdApp.Name+".fly.dev")
+				appConfig.SetEnvVariable(envName, appConfig.AppName+".fly.dev")
 			} else {
 				appConfig.SetEnvVariable(envName, envVal)
 			}
 		}
 
 		if len(srcInfo.Statics) > 0 {
-			appConfig.SetStatics(srcInfo.Statics)
+			var appStatics []app.Static
+			for _, s := range srcInfo.Statics {
+				appStatics = append(appStatics, app.Static{
+					GuestPath: s.GuestPath,
+					UrlPrefix: s.UrlPrefix,
+				})
+			}
+			appConfig.SetStatics(appStatics)
 		}
 
 		if len(srcInfo.Volumes) > 0 {
-			appConfig.SetVolumes(srcInfo.Volumes)
+			var appVolumes []app.Volume
+			for _, v := range srcInfo.Volumes {
+				appVolumes = append(appVolumes, app.Volume{
+					Source:      v.Source,
+					Destination: v.Destination,
+				})
+			}
+			appConfig.SetVolumes(appVolumes)
 		}
 
 		for procName, procCommand := range srcInfo.Processes {
@@ -464,30 +452,75 @@ func run(ctx context.Context) (err error) {
 		if srcInfo.KillSignal != "" {
 			appConfig.SetKillSignal(srcInfo.KillSignal)
 		}
-	}
 
-	// Append any requested Dockerfile entries
-	if srcInfo != nil && len(srcInfo.DockerfileAppendix) > 0 {
-		if err := appendDockerfileAppendix(srcInfo.DockerfileAppendix); err != nil {
-			return fmt.Errorf("failed appending Dockerfile appendix: %w", err)
+		// Append any requested Dockerfile entries
+		if len(srcInfo.DockerfileAppendix) > 0 {
+			if err := appendDockerfileAppendix(srcInfo.DockerfileAppendix); err != nil {
+				return fmt.Errorf("failed appending Dockerfile appendix: %w", err)
+			}
+		}
+
+		if len(srcInfo.BuildArgs) > 0 {
+			if appConfig.Build == nil {
+				appConfig.Build = &app.Build{}
+			}
+			appConfig.Build.Args = srcInfo.BuildArgs
 		}
 	}
 
-	if srcInfo != nil && len(srcInfo.BuildArgs) > 0 {
-		appConfig.Build = &app.Build{}
-		appConfig.Build.Args = srcInfo.BuildArgs
+	if n := flag.GetInt(ctx, "internal-port"); n > 0 {
+		appConfig.SetInternalPort(n)
 	}
 
 	// Finally, write the config
-
-	if err = appConfig.WriteToDisk(ctx, filepath.Join(workingDir, "fly.toml")); err != nil {
+	if err := appConfig.WriteToDisk(ctx, configFilePath); err != nil {
 		return err
 	}
 
+	ctx = app.WithConfig(ctx, appConfig)
+
 	if srcInfo == nil {
 		return nil
 	}
 
+	deployArgs := deploy.DeployWithConfigArgs{
+		ForceNomad:    flag.GetBool(ctx, "force-nomad"),
+		ForceMachines: flag.GetBool(ctx, "force-machines"),
+		ForceYes:      flag.GetBool(ctx, "now"),
+	}
+	var v2AppConfig *appv2.Config
+	if deployArgs.ForceMachines {
+		v2AppConfig, err = appv2.LoadConfig(configFilePath)
+		if err != nil {
+			return fmt.Errorf("invalid config: %w", err)
+		}
+		ctx = appv2.WithConfig(ctx, v2AppConfig)
+	}
+	if deployArgs.ForceMachines && !deployArgs.ForceYes {
+		if !flag.GetBool(ctx, "no-deploy") && !flag.GetBool(ctx, "now") && !flag.GetBool(ctx, "auto-confirm") && v2AppConfig.HasNonHttpAndHttpsStandardServices() {
+			hasUdpService := v2AppConfig.HasUdpService()
+			ipStuffStr := "a dedicated ipv4 address"
+			if !hasUdpService {
+				ipStuffStr = "dedicated ipv4 and ipv6 addresses"
+			}
+			confirmDedicatedIp, err := prompt.Confirmf(ctx, "Would you like to allocate %s now?", ipStuffStr)
+			if confirmDedicatedIp && err == nil {
+				v4Dedicated, err := client.AllocateIPAddress(ctx, v2AppConfig.AppName, "v4", "", nil, "")
+				if err != nil {
+					return err
+				}
+				fmt.Fprintf(io.Out, "Allocated dedicated ipv4: %s\n", v4Dedicated.Address)
+				if !hasUdpService {
+					v6Dedicated, err := client.AllocateIPAddress(ctx, v2AppConfig.AppName, "v6", "", nil, "")
+					if err != nil {
+						return err
+					}
+					fmt.Fprintf(io.Out, "Allocated dedicated ipv6: %s\n", v6Dedicated.Address)
+				}
+			}
+		}
+	}
+
 	// Notices from a launcher about its behavior that should always be displayed
 	if srcInfo.Notice != "" {
 		fmt.Fprintln(io.Out, srcInfo.Notice)
@@ -514,7 +547,7 @@ func run(ctx context.Context) (err error) {
 	}
 
 	if deployNow {
-		return deploy.DeployWithConfig(ctx, appConfig)
+		return deploy.DeployWithConfig(ctx, appConfig, deployArgs)
 	}
 
 	// Alternative deploy documentation if our standard deploy method is not correct
@@ -587,7 +620,6 @@ func appendDockerfileAppendix(appendix []string) (err error) {
 }
 
 func shouldDeployExistingApp(ctx context.Context, appName string) (bool, error) {
-
 	client := client.FromContext(ctx).API()
 	status, err := client.GetAppStatus(ctx, appName, false)
 	if err != nil {
@@ -739,9 +771,9 @@ func determineDockerIgnore(ctx context.Context, workingDir string) (err error) {
 	return
 }
 
-func LaunchPostgres(ctx context.Context, app *api.App, org *api.Organization, region *api.Region) {
+func LaunchPostgres(ctx context.Context, appName string, org *api.Organization, region *api.Region) {
 	io := iostreams.FromContext(ctx)
-	clusterAppName := app.Name + "-db"
+	clusterAppName := appName + "-db"
 	err := postgres.CreateCluster(ctx, org, region,
 		&postgres.ClusterParams{
 			PostgresConfiguration: postgres.PostgresConfiguration{
@@ -754,28 +786,26 @@ func LaunchPostgres(ctx context.Context, app *api.App, org *api.Organization, re
 	} else {
 		err = postgres.AttachCluster(ctx, postgres.AttachParams{
 			PgAppName: clusterAppName,
-			AppName:   app.Name,
+			AppName:   appName,
 		})
 
 		if err != nil {
 			msg := `Failed attaching %s to the Postgres cluster %s: %w.\nTry attaching manually with 'fly postgres attach --app %s %s'`
-			fmt.Fprintf(io.Out, msg, app.Name, clusterAppName, err, app.Name, clusterAppName)
+			fmt.Fprintf(io.Out, msg, appName, clusterAppName, err, appName, clusterAppName)
 
 		} else {
-			fmt.Fprintf(io.Out, "Postgres cluster %s is now attached to %s\n", clusterAppName, app.Name)
+			fmt.Fprintf(io.Out, "Postgres cluster %s is now attached to %s\n", clusterAppName, appName)
 		}
 	}
 }
 
-func LaunchRedis(ctx context.Context, app *api.App, org *api.Organization, region *api.Region) {
-
-	name := app.Name + "-redis"
-
+func LaunchRedis(ctx context.Context, appName string, org *api.Organization, region *api.Region) {
+	name := appName + "-redis"
 	db, err := redis.Create(ctx, org, name, region, "", true, false)
 
 	if err != nil {
 		fmt.Println(fmt.Errorf("%w", err))
 	} else {
-		redis.AttachDatabase(ctx, db, app)
+		redis.AttachDatabase(ctx, db, appName)
 	}
 }
