@@ -5,6 +5,7 @@ import (
 	"fmt"
 
 	"github.com/spf13/cobra"
+	"github.com/superfly/flyctl/api"
 	"github.com/superfly/flyctl/client"
 	"github.com/superfly/flyctl/flaps"
 	"github.com/superfly/flyctl/internal/app"
@@ -102,6 +103,22 @@ func runMachineList(ctx context.Context) (err error) {
 				volName = machine.Config.Mounts[0].Volume
 			}
 
+			appPlatform := ""
+			machineProcessGroup := ""
+
+			if machine.Config != nil {
+				if platformVersion, ok := machine.Config.Metadata[api.MachineConfigMetadataKeyFlyPlatformVersion]; ok {
+					appPlatform = platformVersion
+
+				}
+
+				if processGroup, ok := machine.Config.Metadata[api.MachineConfigMetadataKeyFlyProcessGroup]; ok {
+					machineProcessGroup = processGroup
+
+				}
+
+			}
+
 			rows = append(rows, []string{
 				machine.ID,
 				machine.Name,
@@ -112,10 +129,13 @@ func runMachineList(ctx context.Context) (err error) {
 				volName,
 				machine.CreatedAt,
 				machine.UpdatedAt,
+				appPlatform,
+				machineProcessGroup,
 			})
+
 		}
 
-		_ = render.Table(io.Out, appName, rows, "ID", "Name", "State", "Region", "Image", "IP Address", "Volume", "Created", "Last Updated")
+		_ = render.Table(io.Out, appName, rows, "ID", "Name", "State", "Region", "Image", "IP Address", "Volume", "Created", "Last Updated", "App Platform", "Process Group")
 	}
 	return nil
 }
