@@ -22,7 +22,7 @@ func newDetach() *cobra.Command {
 	const (
 		short = "Detach a postgres cluster from an app"
 		long  = short + "\n"
-		usage = "detach [POSTGRES APP]"
+		usage = "detach <POSTGRES APP>"
 	)
 
 	cmd := command.New(usage, short, long, runDetach,
