@@ -5,6 +5,7 @@ import (
 	"fmt"
 	"net"
 	"os"
+	"time"
 
 	"github.com/AlecAivazis/survey/v2"
 	"github.com/docker/docker/pkg/ioutils"
@@ -308,7 +309,7 @@ func addrForMachines(ctx context.Context, app *api.AppCompact, console bool) (ad
 				return "", err
 			}
 
-			err = flapsClient.Wait(ctx, selectedMachine, "started")
+			err = flapsClient.Wait(ctx, selectedMachine, "started", 60*time.Second)
 
 			if err != nil {
 				return "", err
